{******************************************************************************}
{ Projeto: ACBr Monitor                                                        }
{  Executavel multiplataforma que faz uso do conjunto de componentes ACBr para }
{ criar uma interface de comunicao com equipamentos de automacao comercial.  }
{                                                                              }
{ Direitos Autorais Reservados (c) 2006 Daniel Simoes de Almeida               }
{                                                                              }
{ Colaboradores nesse arquivo:                                                 }
{                                                                              }
{  Voc pode obter a ltima verso desse arquivo na pgina do Projeto ACBr     }
{ Componentes localizado em      http://www.sourceforge.net/projects/acbr      }
{                                                                              }
{  Este programa  software livre; voc pode redistribu-lo e/ou modific-lo   }
{ sob os termos da Licena Pblica Geral GNU, conforme publicada pela Free     }
{ Software Foundation; tanto a verso 2 da Licena como (a seu critrio)       }
{ qualquer verso mais nova.                                                   }
{                                                                              }
{  Este programa  distribudo na expectativa de ser til, mas SEM NENHUMA     }
{ GARANTIA; nem mesmo a garantia implcita de COMERCIALIZAO OU DE ADEQUAO A}
{ QUALQUER PROPSITO EM PARTICULAR. Consulte a Licena Pblica Geral GNU para  }
{ obter mais detalhes. (Arquivo LICENCA.TXT ou LICENSE.TXT)                    }
{                                                                              }
{  Voc deve ter recebido uma cpia da Licena Pblica Geral GNU junto com este}
{ programa; se no, escreva para a Free Software Foundation, Inc., 59 Temple   }
{ Place, Suite 330, Boston, MA 02111-1307, USA. Voc tambm pode obter uma     }
{ copia da licena em:  http://www.opensource.org/licenses/gpl-license.php     }
{                                                                              }
{ Daniel Simes de Almeida  -  daniel@djsystem.com.br  -  www.djsystem.com.br  }
{              Praa Anita Costa, 34 - Tatu - SP - 18270-410                  }
{                                                                              }
{******************************************************************************}
unit CmdUnitNFe;

interface
Uses SysUtils, Classes, Math ;

Const
   Objetos = '"NFE"' ;

type
TACBrNFeCmd = class
  private
    fsParams : TStringList ;
    fsComando: WideString;
    fsObjeto : String;
    fsMetodo : String;
    fsResposta: WideString;
    procedure SetComando(const Value: WideString);

  public
    constructor Create;
    destructor Destroy; override ;

    function Params( Index : Integer) : String ;

    property Comando : WideString  read fsComando write SetComando ;
    property Objeto  : String      read fsObjeto ;
    property Metodo  : String      read fsMetodo ;
    property Resposta: WideString  read fsResposta write fsResposta ;
  end;

implementation

uses StrUtils;

{----------------------------------- TACBrCmd ---------------------------------}
constructor TACBrNFeCmd.Create;
begin
  fsParams := TStringList.Create ;
end;

destructor TACBrNFeCmd.Destroy;
begin
  fsParams.Free ;
  inherited Destroy ;
end;

function TACBrNFeCmd.Params(Index: Integer): String;
begin
  if Index > fsParams.Count-1 then
     Result := ''
  else
     Result := fsParams[Index] ;
end;

procedure TACBrNFeCmd.SetComando(const Value: WideString);
Var P,PaI,PaF,Pv : Integer ;
    wComando, wParam, wProxChar : WideString ;
begin
  fsMetodo   := '' ;
  fsObjeto   := '' ;
  fsResposta := '' ;
  fsParams.Clear ;

  fsComando := Value ;
  wComando  := Value ;

  { Achando o Objeto }
  P := pos('.',wComando) ;
  if P = 0 then
     raise Exception.Create('Objeto nao definido') ;

  fsObjeto := UpperCase( Trim(copy(fsComando,1,P-1)) ) ;
  if pos('"'+fsObjeto+'"', Objetos) = 0 then
     raise Exception.Create('Objeto invlido: '+fsObjeto+sLineBreak+
                            ' Permitidos: '+Objetos ) ;

  wComando := copy(wComando, P+1, Length(wComando) ) ;

  { Achando o Mtodo }
  P := pos('(',wComando) ;
  if P = 0 then
  begin
     { Verificando se  uma atribuio a propriedade Ex: "Ativo := true"
       Se for, transforma em "SetAtivo(True)"  }
     P := pos(':=',wComando) ;
     if P > 0 then
      begin
         wComando := 'Set'+Trim(copy(wComando,1,P-1))+'('+
                           Trim(copy(wComando,1,P+2))+')' ;
         P := pos('(',wComando) ;
      end
     else
        P := Length( wComando ) + 1 ;
  end ;

  fsMetodo := LowerCase( Trim(copy(wComando,1,P-1)) ) ;
  if fsMetodo = '' then
     raise Exception.Create('Metodo no informado') ;

  { Tem Parameteros ? }
  wComando := copy(wComando, P+1, Length(wComando) ) ;
  while Length(wComando) > 0 do
  begin
     PaI := pos('"',wComando) ;
     Pv  := pos(',',wComando) ;  { Procurando o Fim do parametro }
     if Pv = 0 then
        Pv := pos(')',wComando) ;
     if Pv = 0 then
        Pv := Length( wComando ) + 1;

     if (PaI <> 0) and (PaI < Pv) then  { Tem aspas no Inicio do Comando }
      begin                             { Entao procure a proxima Aspas }
       { Verificando se a aspas  seguida de ',' ou ')' ou fim do comando
          Se no for, ento a aspas no  um delimitador de String e sim
          faz parte do texto da String }
        PaF := PaI ;
        wProxChar := ' ' ;
        while (PaF <> 0) and
              (wProxChar <> '') and
              (pos(wProxChar, ',)') = 0) do
        begin
           if wProxChar = '"' then
              Inc( PaF ) ;
           PaF := PaF + max(Pos('"',copy(wComando, PaF+1, Length(Wcomando)) ),1) ;
           wProxChar := copy(Trim(copy(wComando, PaF+1 , Length(Wcomando))),1,1) ;
        end ;

        if PaF = 0 then
           raise Exception.Create('Parametro Invlido. String no terminada');

        wParam := copy(wComando, PaI+1 , PaF-PaI-1 ) ;
        Pv := PosEx(',', wComando, PaF+1 ) ;
        if Pv = 0 then
           Pv := Length( wComando ) + 1;
      end
     else
        wParam := copy(wComando, 1, Pv-1 ) ;

     { convertendo as aspas duplas "", para simples " }
     wParam := StringReplace(wParam,'""','"',[rfReplaceAll]) ;

     fsParams.Add( wParam ) ;
     wComando := copy(wComando, Pv+1, Length( wComando ) ) ;
  end ;
end;

end.
 