{******************************************************************************}
{ Projeto: ACBr Monitor                                                        }
{  Executavel multiplataforma que faz uso do conjunto de componentes ACBr para }
{ criar uma interface de comunicao com equipamentos de automacao comercial.  }
{                                                                              }
{ Direitos Autorais Reservados (c) 2006 Daniel Simoes de Almeida               }
{                                                                              }
{ Colaboradores nesse arquivo:                                                 }
{                                                                              }
{  Voc pode obter a ltima verso desse arquivo na pgina do Projeto ACBr     }
{ Componentes localizado em      http://www.sourceforge.net/projects/acbr      }
{                                                                              }
{  Este programa  software livre; voc pode redistribu-lo e/ou modific-lo   }
{ sob os termos da Licena Pblica Geral GNU, conforme publicada pela Free     }
{ Software Foundation; tanto a verso 2 da Licena como (a seu critrio)       }
{ qualquer verso mais nova.                                                   }
{                                                                              }
{  Este programa  distribudo na expectativa de ser til, mas SEM NENHUMA     }
{ GARANTIA; nem mesmo a garantia implcita de COMERCIALIZAO OU DE ADEQUAO A}
{ QUALQUER PROPSITO EM PARTICULAR. Consulte a Licena Pblica Geral GNU para  }
{ obter mais detalhes. (Arquivo LICENCA.TXT ou LICENSE.TXT)                    }
{                                                                              }
{  Voc deve ter recebido uma cpia da Licena Pblica Geral GNU junto com este}
{ programa; se no, escreva para a Free Software Foundation, Inc., 59 Temple   }
{ Place, Suite 330, Boston, MA 02111-1307, USA. Voc tambm pode obter uma     }
{ copia da licena em:  http://www.opensource.org/licenses/gpl-license.php     }
{                                                                              }
{ Daniel Simes de Almeida  -  daniel@djsystem.com.br  -  www.djsystem.com.br  }
{              Praa Anita Costa, 34 - Tatu - SP - 18270-410                  }
{                                                                              }
{******************************************************************************}

{$DEFINE VisualCLX}
{$I ACBr.inc}

unit DoACBrUnit ;

interface
Uses Classes, TypInfo, SysUtils, CmdUnit, Types ;
{$IFDEF MSWINDOWS}
  function BlockInput (fBlockInput: boolean): dword; stdcall; external 'user32.dll';
{$ENDIF}

Procedure DoACBr( Cmd : TACBrCmd ) ;
Procedure VerificaPermiteComandosRemoto ;
Function ConvertStrRecived( AStr: String ) : String ;

implementation
Uses ACBrUtil, StrUtils, DateUtils,
  {$IFDEF MSWINDOWS}sndkey32, Windows,{$ENDIF}
  {$IFNDEF CONSOLE}QForms, ACBrMonitor1 {$ELSE}ACBrMonitorConsoleDM {$ENDIF} ;

Procedure DoACBr( Cmd : TACBrCmd ) ;
Var AltTab : Boolean ;
    Memo   : TStringList ;
    dtFim  : TDateTime ;
    nWait  : Integer ;
    Files  : String ;
begin
  with {$IFNDEF CONSOLE}FrmACBrMonitor{$ELSE}dm{$ENDIF} do
  begin
     try
        if Cmd.Metodo = 'run' then
         begin
           VerificaPermiteComandosRemoto ;

           if Cmd.Params(0) = '' then
              raise Exception.Create('Linha de comando no informada');

           AltTab := StrToBoolDef(Cmd.Params(4),False) ; { Envia ALT-TAB quando Terminar ?}
           RunCommand( Cmd.Params(0),                       { Linha de comando }
                       Cmd.Params(1),                       { Parametros adicionais }
                       StrToBoolDef(Cmd.Params(2),False), { Aguarda termino execuao ? }
                       StrToIntDef(Cmd.Params(3),1));
           {$IFDEF MSWINDOWS}
           if AltTab then
              SendKeys(pchar('%{TAB}'), False);
           {$ENDIF}
         end

        {$IFNDEF CONSOLE}
          else if Cmd.Metodo = 'restaurar' then
             Restaurar1Click( FrmACBrMonitor )

          else if Cmd.Metodo = 'ocultar' then
             Ocultar1Click( FrmACBrMonitor )

          else if Cmd.Metodo = 'encerrarmonitor' then
             Application.Terminate
        {$ENDIF}

        {$IFDEF MSWINDOWS}
         else if Cmd.Metodo = 'sendkeys' then
            SendKeys( PChar(Cmd.Params(0)),                  { Teclas a Enviar }
                      StrToBoolDef(Cmd.Params(1),False) )           { Espera ? }
            
         else if Cmd.Metodo = 'appactivate' then
          begin
            nWait := StrToIntDef( Cmd.Params(1), 0 ) ;
            if nWait > 0 then
               Sleep(nWait);
            AppActivate( PChar(Cmd.Params(0)) ) ;
          end

         else if Cmd.Metodo = 'appexists' then
            Cmd.Resposta := BoolToStr( AppExists( PChar(Cmd.Params(0)) ), true )

         else if Cmd.Metodo = 'blockinput' then
           BlockInput( StrToBool(Cmd.Params(0)) )
        {$ENDIF}

        else if Cmd.Metodo = 'savetofile' then
         begin
           VerificaPermiteComandosRemoto ;

           Memo := TStringList.Create ;
           try
              Memo.Clear ;
              Memo.Text := ConvertStrRecived( cmd.Params(1) );
              Memo.SaveToFile( Cmd.Params(0) );
           finally
              Memo.Free ;
           end ;
         end

        else if Cmd.Metodo = 'loadfromfile' then
         begin
           VerificaPermiteComandosRemoto ;

           Files := Cmd.Params(0) ;
           dtFim := IncSecond(now, StrToIntDef(Cmd.Params(1),1) ) ;
           while now <= dtFim do
           begin
              if FileExists( Files ) then
              begin
                 Memo  := TStringList.Create ;
                 try
                    Memo.Clear ;
                    Memo.LoadFromFile( Files ) ;
                    Cmd.Resposta := Memo.Text ;
                    Break ;
                 finally
                    Memo.Free ;
                 end ;
              end ;

              {$IFNDEF CONSOLE}
               Application.ProcessMessages ;
              {$ENDIF}
              sleep(100) ;
           end ;

           if not FileExists( Cmd.Params(0) ) then
              raise Exception.Create('Arquivo '+Cmd.Params(0)+' no encontrado')
         end

        else if Cmd.Metodo = 'filesexists' then
         begin
           VerificaPermiteComandosRemoto ;

           Files := Cmd.Params(0) ;
           dtFim := IncSecond(now, StrToIntDef(Cmd.Params(1),0) ) ;
           while (now <= dtFim) and ( not FileExists( Files ) ) do
           begin
              {$IFNDEF CONSOLE}
               Application.ProcessMessages ;
              {$ENDIF}
              sleep(100) ;
           end ;

           Cmd.Resposta := BoolToStr(FilesExists( Files ), True) ;
         end

        else if Cmd.Metodo = 'deletefiles' then
         begin
           VerificaPermiteComandosRemoto ;

           Files := Trim(Cmd.Params(0)) ;
           if (Files = '') or (Files = '*') or (Files = '*.*') then
              raise Exception.Create( 'Mascara invlida: ['+Files+']') ;

           DeleteFiles( Files ) ;
           if FilesExists( Files ) then
              raise Exception.Create('Arquivo(s) ['+Files+'] ainda existe(m)') ;
         end

        else if pos('|'+Cmd.Metodo+'|', '|exit|bye|fim|sair|') > 0 then {fecha conexao}
         begin
           Cmd.Resposta := 'Obrigado por usar o ACBrMonitor' ;
           {$IFNDEF CONSOLE}
             mCmd.Lines.Clear;
           {$ELSE}
             WriteLn( 'Obrigado por usar o ACBrMonitorConsole' ) ;
           {$ENDIF}

           if Assigned( Conexao ) then
              if Assigned( Conexao.Connection ) then
                 Conexao.Connection.Disconnect ;
         end

        ELSE
           raise Exception.Create('Comando invlido ('+ copy(Cmd.Comando,6,length(Cmd.Comando))+')') ;

     finally
        { Nada a fazer aqui por enquanto... :) }
     end ;
  end ;
end ;

Procedure VerificaPermiteComandosRemoto ;
begin
  with {$IFNDEF CONSOLE}FrmACBrMonitor{$ELSE}dm{$ENDIF} do
  begin
     {$IFNDEF CONSOLE}
       if not cbComandos.Checked then
     {$ELSE}
       if not PermiteComandos then
     {$ENDIF}
          raise Exception.Create('Comandos Remotos no so permitidos');
  end ;
end ;

Function ConvertStrRecived( AStr: String ) : String ;
 Var P   : Integer ;
     Hex : String ;
     CharHex : Char ;
begin
  { Verificando por codigos em Hexa }
  Result := AStr ;

  P := pos('\x',Result) ;
  while P > 0 do
  begin
     Hex := copy(Result,P+2,2) ;

     try
        CharHex := Chr(StrToInt('$'+Hex)) ;
     except
        CharHex := ' ' ;
     end ;

     Result := StringReplace(Result,'\x'+Hex,CharHex,[rfReplaceAll]) ;
     P      := pos('\x',Result) ;
  end ;
end ;

end.


